import type { NextApiRequest, NextApiResponse } from 'next';
/**
 *  public builder method of RouterBuilder that accepts next.js handler
 *  and return RouterBuilder itself
 */
export declare type RouterHandler<T = unknown> = (handler: NextApiHandler<T>) => RouterBuilder;
/**
 *  a sync/async function that takes next.js request and optional next.js response
 */
export declare type NextApiHandler<T = unknown> = (req: NextApiRequestWithMiddleware, res?: NextApiResponse) => T | Promise<T>;
/**
 *  a next.js api request with injected req.middleware
 */
export interface NextApiRequestWithMiddleware<T = unknown> extends NextApiRequest {
    middleware: Record<string, T>;
}
/**
 *  a standard next.js api handler but with req.middleware available
 *  see [official doc](https://nextjs.org/docs/api-routes/introduction) for more details
 */
export declare type NextApiHandlerWithMiddleware<T = unknown, M = unknown> = (req: NextApiRequestWithMiddleware<M>, res: NextApiResponse<T>) => void | Promise<void>;
/**
 *  all api response combines success response & error response
 */
export declare type ApiResponse<T = unknown> = SuccessApiResponse<T> | ErrorApiResponse;
/**
 *  api response that has http status code 2xx with data
 */
export declare type SuccessApiResponse<T> = {
    success: true;
    data: T;
};
/**
 *  api response that has http status code 4xx or 5xx with error message
 */
export declare type ErrorApiResponse = {
    success: false;
    message: string;
};
/**
 * A router builder is next api handler builder that expose express-like api.
 *
 * ### Example (basic usage)
 * ```js
 * // In next.js /pages/api/[route]
 * import { RouterBuilder } from 'next-api-handler'
 * const handler = new RouterBuilder().build()
 * export default handler
 * ```
 *
 * ### Example (add RESTful method)
 * ```js
 * import { RouterBuilder } from 'next-api-handler'
 * const handler = new RouterBuilder().get((req, res) => "RESPONSE")
 * export default handler
 * ```
 *
 * ### Example (add async method)
 * ```js
 * import { RouterBuilder } from 'next-api-handler'
 * const handler = new RouterBuilder().get(async (req, res) => "ASYNC RESPONSE")
 * export default handler
 * ```
 *
 * @returns a builder that can build a next.js api handler.
 */
export declare class RouterBuilder {
    private readonly route;
    private readonly middlewareList;
    private readonly middlewareQueue;
    get<T = unknown>(handler: NextApiHandler<T>): RouterBuilder;
    head<T = unknown>(handler: NextApiHandler<T>): RouterBuilder;
    patch<T = unknown>(handler: NextApiHandler<T>): RouterBuilder;
    options<T = unknown>(handler: NextApiHandler<T>): RouterBuilder;
    connect<T = unknown>(handler: NextApiHandler<T>): RouterBuilder;
    delete<T = unknown>(handler: NextApiHandler<T>): RouterBuilder;
    trace<T = unknown>(handler: NextApiHandler<T>): RouterBuilder;
    post<T = unknown>(handler: NextApiHandler<T>): RouterBuilder;
    put<T = unknown>(handler: NextApiHandler<T>): RouterBuilder;
    use<T extends Record<string, unknown> = Record<string, unknown>>(handler: NextApiHandler<T>): RouterBuilder;
    inject<T extends Record<string, unknown> = Record<string, unknown>>(handler: NextApiHandler<T>): RouterBuilder;
    build(): NextApiHandlerWithMiddleware;
    private handleMiddlewareQueue;
    private handleMiddlewareList;
    private add;
}
